@@ -1,13 +1,13 @@
 import os
 from tempfile import TemporaryDirectory
 
-from mcp_scan.StorageFile import StorageFile
+from mcp_scan.Storage import Storage
 
 
 def test_whitelist():
     with TemporaryDirectory() as tempdir:
         path = os.path.join(tempdir, "storage.json")
-        storage_file = StorageFile(path)
+        storage_file = Storage(path)
         storage_file.add_to_whitelist("tool", "test", "test")
         storage_file.add_to_whitelist("tool", "test", "test2")
         storage_file.add_to_whitelist("tool", "asdf", "test2")
@@ -19,7 +19,7 @@ def test_whitelist():
         storage_file.save()
 
         # test reload
-        storage_file = StorageFile(path)
+        storage_file = Storage(path)
         assert len(storage_file.whitelist) == 2
 
         # test reset
